#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define NO_SUCCESS_MESSAGES 5
const char SUCCESS_MESSAGES[][100] = {"Extraordinar", "Fabulos", "Exceptional", "Mirific", "WoW"};

#define NO_FAILURE_MESSAGES 5
const char FAILURE_MESSAGES[][100] = {"Nu prea e bine", "Cam nasol momentu'", "Mai incearca", "Pfff", "Ghinion"};

const char* getSuccessMessage() {
  return SUCCESS_MESSAGES[rand() % NO_SUCCESS_MESSAGES];
}
const char* getFailureMessage() {
  return FAILURE_MESSAGES[rand() % NO_FAILURE_MESSAGES];
}

#define MAX_S 200000

char postfix[MAX_S];
int postfixPointer;

char operators[MAX_S];
int operatorsSize;

void pushToPostfix(char ch) {
  if (postfixPointer < MAX_S)
    postfix[postfixPointer++] = ch;
}

int priority(char op) {
  if (op == '*' || op == '/')
    return 2;
  if (op == '+' || op == '-')
    return 1;
  return 0;
}
char topOperator() {
  return operatorsSize > 0 ? operators[operatorsSize - 1] : 0;
}
char popOperator() {
  return operatorsSize > 0 ? operators[--operatorsSize] : 0;
}
void pushOperator(char ch) {
  if (operatorsSize < MAX_S)
    operators[operatorsSize++] = ch;
}

bool isOperand(char ch) {
  return ch >= 'A' && ch <= 'Z';
}
bool isOperator(char ch) {
  return ch == '+' || ch == '-' || ch == '*' || ch == '/';
}
bool isParenthesis(char ch) {
  return ch == '(' || ch == ')';
}
bool isCharValid(char ch) {
  return isOperand(ch) || isOperator(ch) || isParenthesis(ch);
}

int main() {
  srand(time(0));

  FILE *fin, *fout;
  int noCharsProcessed, postfixLength;
  char ch;

  noCharsProcessed = 0;

  fin = fopen("postinfix.out", "r");
  if (fin != NULL) {
    ch = fgetc(fin);
    while (noCharsProcessed++ < MAX_S && isCharValid(ch)) {
      if (ch >= 'A' && ch <= 'Z')
        pushToPostfix(ch);
      else if (ch == '(')
        pushOperator(ch);
      else if (ch == ')') {
        while (topOperator() && topOperator() != '(')
          pushToPostfix(popOperator());
        popOperator();
      } else if (priority(ch)) {
        while (topOperator() && priority(topOperator()) >= priority(ch))
          pushToPostfix(popOperator());
        pushOperator(ch);
      }

      ch = fgetc(fin);
    }

    while (topOperator())
      pushToPostfix(popOperator());

    fclose(fin);

    fout = fopen("postinfix.in", "r");

    postfixLength = postfixPointer;
    postfixPointer = 0;
    ch = fgetc(fout);
    while (postfixPointer < postfixLength && postfix[postfixPointer] == ch) {
      ++postfixPointer;
      ch = fgetc(fout);
    }

    if (postfixPointer == postfixLength && ch == '\n') {
      fprintf(stderr, getSuccessMessage());
      printf("5");
    } else {
      fprintf(stderr, getFailureMessage());
      printf("0");
    }

    fclose(fout);
  } else {
    fprintf(stderr, "Of... Fisierul de iesire nu e bun!");
    printf("0");
  }

  return 0;
}
